/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rational.report.web.core.app.view.navigation.ReportNodeParser");

/**
 * Parses the report tree nodes.
 */
com.ibm.rational.report.web.core.app.view.navigation.ReportNodeParser = function(data) {

	this.raw_data = null;
	
	// map of node and children
	this.nodeMap = [];
	this.nodeChildrenMap = [];
	this.rootNodes = [];
	
	this.refresh = function(data) {
		this.raw_data = data;
		if (this.raw_data == null) {
			this.raw_data = [];
		}
		
		// map of node and children
		this.nodeMap = [];
		this.nodeChildrenMap = [];
		this.rootNodes = [];
		
		this._parseData();
	};
	
	this._parseData = function() {
		if (this.raw_data == null) {
			return;
		}
		
		for (var i in this.raw_data) {
			var node = this.raw_data[i];
			this.nodeMap[node.itemId] = node;
			
			if (node.parent == null) {
				this.rootNodes.push(node);
			} else {			
				var children = this.nodeChildrenMap[node.parent.itemId];
				if (!children) {
					children = [];
					this.nodeChildrenMap[node.parent.itemId] = children;
				}
				children.push(node);
			}
		}
	};
	
	this.getNode = function(id) {
		if (id == null) {
			return null;
		}
		return this.nodeMap[id];
	};
	
	this.getAllNodes = function() {
		return this.raw_data;
	};
	
	this.getReportNode = function(parentId, reportId) {
		var nodes = this.getChildren(parentId);	
		if (nodes == null) return null;
		
		for (var i in nodes) {
			var node = nodes[i];
			if (node.report && node.report.itemId == reportId) {
				return node;
			}
		}
		
		return null;
	};
	
	this.getRootNodes = function() {
		return this.rootNodes;
	};
	 
	this.getChildren = function(nodeId) {
		var nodes = [];
		for (var i in this.raw_data) {
			var node = this.raw_data[i];
			if (node.parent != null && node.parent.itemId == nodeId) {
				nodes.push(node);
			}
		}
		
		return nodes;
	};
	
	this.getAllChildren = function(nodeId, result) {
		var nodes = this.nodeChildrenMap[nodeId];
		if ( nodes != null && nodes.length > 0 ) {
			result = result.concat(nodes);
			
			for ( var i in nodes ) {
				var node = nodes[i];
				result = this.getAllChildren(node.itemId, result);
			}	
		}
				
		return result;
	};
	
	this.getValidParents = function(nodeId) {
		if (nodeId == null) {
			return this.raw_data;
		}
		
		var theNode = this.getNode(nodeId);
		if (theNode.report) {
			return this.raw_data;
		}
		
		var nodes = [];
		for (var i in this.raw_data) {
			var node = this.raw_data[i];
			if (node == theNode) continue;

			var n = null;
			if (node.parent != null) {
				n = this.getNode(node.parent.itemId);
				while(n != null && n.itemId != nodeId) {
					n = n.parent;
					if (n != null) n = this.getNode(n.itemId);
				}
			}
				
			if ((n == null || n.itemId != nodeId) && node.shared == theNode.shared) {
				nodes.push(node);
			}
		}
		
		return nodes;
	};
	
	// parse the data 
	this.refresh(data);
		
};
